//  CRC.h
//  Network Simulator
//
//  Created by Wade Tregaskis on Sat Apr 03 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// Yeah, I suppose I should explain this... somewhere deep in the system headers CRC_H is already in use by something else.  Woot seems as good a midfix as any. :P
#ifndef CRC_WOOT_H
#define CRC_WOOT_H

#include <stdlib.h>
#include <stdio.h>


#ifndef UInt16
typedef u_int16_t UInt16; // UInt16 is apparently a Mac/BSD-specific type, so it's not present on Red Hat boxes
#endif

#ifndef UInt32
typedef u_int32_t UInt32; // UInt32 is apparently a Mac/BSD-specific type, so it's not present on Red Hat boxes
#endif

/* @function crc16
   @abstract Updates a running CRC16 calculation with more data.
   @discussion The CRC of a large segment of data is calculated by iterating over it in 16-bit chunks and updating the CRC at each step.  That is what this function does.  It is not a very fast CRC implementation, not using lookup tables or anything, but so far as CRC implementations go it is pretty good.
   @param crc The current CRC value.  If you are just starting the CRC calculation, you may use any value you like, so long as you start with that value again when verifying the CRC.  Generally 0 is an intelligent choice.
   @param value The value to update the CRC with.
   @result The new CRC. */

UInt16 crc16(UInt16 crc, UInt16 value);

/*! @function crc16OfString
    @abstract Calculates the CRC16 of a length of data.
    @discussion This function iterates over the length of the given data, calculating the CRC for it.  The CRC is initially assumed to be 0.  If there are an odd number of bytes in the string, the odd-byte-out is shifted to the higher-order position of the 16-bit chunk and then added to the CRC.
    @param string The data to calculate the CRC for.  The name 'string' may be misleading - the data will be considered to be of the given length, regardless of the occurance of any NULL characters within it.
    @param length The length of the given data, which must be greater than or equal to 0.
    @result Returns the CRC16 of the given data. */

UInt16 crc16OfString(const char *string, int length);

#endif
