//  NetworkLinkDropper.cpp
//  Network Simulator
//
//  Created by Wade Tregaskis on Wed Apr 21 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "NetworkLinkDropper.h"


NetworkLinkDropper::NetworkLinkDropper(Network *home, int transmission, float dropProb) : NetworkLink(home, transmission), dataDropProbability(dropProb) {
    this->addType(NETWORK_LINK_DROPPER);
    
    assert(dataDropProbability >= 0.0);
}

NetworkLinkDropper::~NetworkLinkDropper() {
    // Do nothing
}

void NetworkLinkDropper::submitData(const char *data, unsigned int length, NetworkEntity *source, bool &ownership) {
    if (RANDOM(0.0, 1.0) > dataDropProbability) {
        this->NetworkLink::submitData(data, length, source, ownership);
    } else {
        printf("(%s) NetworkLinkDropper[%llu] dropping data.\n", this->className(), id);
    }
}

const char* NetworkLinkDropper::className(void) const {
    return "NetworkLinkDropper";
}

void NetworkLinkDropper::outputClassHierarchy(ostream &out) const {
    this->NetworkLink::outputClassHierarchy(out);
    out << " -> " << this->NetworkLinkDropper::className();
}

void NetworkLinkDropper::getDescription(ostream &out) const {
    this->NetworkLink::getDescription(out);
    out << "Data drop probability: " << dataDropProbability << '\n';
}

void NetworkLinkDropper::encode(Coder *coder) const {
    this->NetworkLink::encode(coder);
    
    coder->encodeFloat(dataDropProbability);
}

bool NetworkLinkDropper::decode(Decoder *decoder) {
    if (this->NetworkLink::decode(decoder)) {
        dataDropProbability = decoder->decodeFloat();
        
        return true;
    } else {
        return false;
    }
}
