//  NetworkLinkDropper.h
//  Network Simulator
//
//  Created by Wade Tregaskis on Wed Apr 21 2004.
//
//  Copyright (c) 2004, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef NETWORK_LINK_DROPPER_H
#define NETWORK_LINK_DROPPER_H

#include "NetworkLink.h"


#define NETWORK_LINK_DROPPER 31


/*! @class NetworkLinkDropper
    @abstract Represents a lossy network link which randomly drops data completely.
    @discussion Acts exactly like a normal NetworkLink with the one significant exception - there is a certain [configurable] chance that each submitted set of data will be dropped entirely.  This represents a lossy network where attentuation or some other factor plays a significant, probabilistic part.
    */

class NetworkLinkDropper : public NetworkLink {
protected:
    float dataDropProbability; // The probability that a given set of data will be dropped, on a scale of 0.0 (no chance) to 1.0 (certainty).
    
public:
/*! @function NetworkLinkDropper
    @abstract Designated initialiser for the NetworkLinkDropper class.
    @discussion Initialises the receiver in the given network with the given properties.
    @param transmission The transmission time for data across the link.  A transmission time of 0 has the link pass all data as soon as it is received (i.e. in the same tick).  A transmission time of 1 has it pass data out on the tick after it receives it, etc etc.  Must be greater than or equal to 0.
    @param dropProb The probability that a given data submission will be dropped entirely.  Must be greater than 0.0.  0.0 represents no chance (i.e. the receiver acts exactly like a normal, loss-less NetworkLink) while 1.0 represents certainty (every single bit of data will be dropped regardless). */
    
    NetworkLinkDropper(Network *home = NULL, int transmission = 0, float dropProb = 0.0);
    ~NetworkLinkDropper(); // Standard block box destructor... nothing to see here.
    
    // NetworkEntity
    
    void submitData(const char *data, unsigned int length, NetworkEntity *source, bool &ownership);
    
    // Object
    
    const char* className(void) const;
    void outputClassHierarchy(ostream &out) const;
    void getDescription(ostream &out) const;
    void encode(Coder *coder) const;
    bool decode(Decoder *decoder);
};

#endif
