//  Object.cpp
//
//  IMPORTANT: This code was developed by Wade Tregaskis under the employment of the Department of Primary Industries, Victoria, whom reserve all rights to it.  It has been used in good faith by Wade Tregaskis without explicit permission, but with the understanding from the employer that it's private use is acceptable in good faith.
//
//  The underlying message being, don't give this to anyone please as I don't have the rights for it's distribution.  Also, although I did write every character of it, I did so outside of university and as such I understand it may not be eligible for inclusion in university marking and other evaluation.
//
//  P.S. Half this code is redundant for this project, anyway.  I've included it in it's entirety only for completeness.

#include "Object.h"


/*! @header Object
    @abstract Contains the Object, Coder and Decoder classes.
    */

void Coder::encodeBool(bool value) {
#ifdef DEBUG_CODING
    cout << "Coder::encodeBool(" << value << ')' << endl;
#endif
    
    char temp = (char)value;
    
    output.write(&temp, sizeof(char));
}

void Coder::encodeInt(int value) {
#ifdef DEBUG_CODING
    cout << "Coder::encodeInt(" << value << ')' << endl;
#endif
    
    const unsigned int BUFFER_SIZE = sizeof(int);
    char buffer[BUFFER_SIZE];
    
    *((int*)buffer) = value;
    
    output.write(buffer, BUFFER_SIZE);
}

void Coder::encodeFloat(float value) {
#ifdef DEBUG_CODING
    cout << "Coder::encodeFloat(" << value << ')' << endl;
#endif
    
    const unsigned int BUFFER_SIZE = sizeof(float);
    char buffer[BUFFER_SIZE];
    
    *((float*)buffer) = value;
    
    output.write(buffer, BUFFER_SIZE);
}

void Coder::encodeString(const char *value) {
#ifdef DEBUG_CODING
    cout << "Coder::encodeString(" << value << ')' << endl;
#endif
    
    if (value) {
        this->encodeData(value, strlen(value));
    } else {
        this->encodeInt(-1);
    }
}

void Coder::encodeData(const char *data, int length) {
#ifdef DEBUG_CODING
    cout << "Coder::encodeData(" << data << ", " << length << ')' << endl;
#endif
    
    assert(length >= 0);
    
    this->encodeInt(length);
    
    if (length > 0) {
        assert(data);

        output.write(data, length);
    }
}

void Coder::encodeObject(const Object *value) {
#ifdef DEBUG_CODING
    cout << "[Coder::encodeObject(" << (int)value << ")]" << endl;
#endif
    
    this->encodeInt((int)value);
    
    if ((value) && (encodedObjects.insert(value).second)) {
        streampos start, end;
        
        this->encodeInt(value->memberClass());
        
        start = output.tellp();
        this->encodeInt(0); // Placeholder
        
        value->encode(this);
        
        end = output.tellp();
        
        output.seekp(start);
        this->encodeInt(end - start);
        
        output.seekp(end);
    }
}

bool Decoder::decodeBool(void) {
    char temp;
    
    input.read(&temp, sizeof(char));
    
#ifdef DEBUG_DECODING
    cout << "Decoder::decodeBool() == " << (bool)temp << endl;
#endif
    
    return (bool)temp;
}

int Decoder::decodeInt(void) {
    const unsigned int BUFFER_SIZE = sizeof(int);
    char buffer[BUFFER_SIZE];
    
    input.read(buffer, BUFFER_SIZE);
    
#ifdef DEBUG_DECODING
    cout << "Decoder::decodeInt() == " << *((int*)buffer) << endl;
#endif
    
    return *((int*)buffer);
}

float Decoder::decodeFloat(void) {
    const unsigned int BUFFER_SIZE = sizeof(float);
    char buffer[BUFFER_SIZE];
    
    input.read(buffer, BUFFER_SIZE);

#ifdef DEBUG_DECODING
    cout << "Decoder::decodeFloat() == " << *((float*)buffer) << endl;
#endif
    
    return *((float*)buffer);
}

char* Decoder::decodeString(void) {
    int length = this->decodeInt();
    char *data = NULL;
    
    if (length >= 0) {
        data = (char*)malloc(length + 1);
        
        input.read(data, length);
        
        data[length] = '\0';
    }
    
#ifdef DEBUG_DECODING
    cout << "Decoder::decodeString() == " << (data ? data : "NULL") << endl;
#endif
    
    return data;
}

char* Decoder::decodeData(int &length) {
    char *data = NULL;

    length = this->decodeInt();
    
    if (length > 0) {
        data = (char*)malloc(length);
        
        input.read(data, length);
    }
    
#ifdef DEBUG_DECODING
    cout << "Decoder::decodeData() == " << (data ? data : "NULL") << endl;
#endif
    
    return data;
}

Object* Decoder::decodeObject(void) {
    void *id = (void*)(this->decodeInt());
    Object* result = NULL;
    
#ifdef DEBUG_DECODING
    cout << "[Decoder::decodeObject(" << (int)id << ")]" << endl;
#endif
    
    if (id) {
        map<void*, Object*>::const_iterator iter = decodedObjects.find(id);
        result = ((decodedObjects.end() == iter) ? NULL : (*iter).second);//decodedObjects[id];
            
            if (result) {
                RETAIN(result); // Add an extra retain for our new owner, the claimant
            } else {
                pair<void*, Object*> newEntry;
                int type = this->decodeInt();
                
                result = static_cast<Object*>(objectMaker(type));
                
                newEntry.first = id;
                newEntry.second = result;
                decodedObjects.insert(newEntry);
                
                if (result) {
                    streampos start, end, supposedSize;
                    
                    start = input.tellg();
                    
                    supposedSize = this->decodeInt();
                    
                    result->decode(this);
                    
                    end = input.tellg();
                    
                    if ((end - start) != supposedSize) {
                        RELEASE(result);
                        result = NULL;
                        cerr << "Unable to decode object type " << type << " at position " << start << " because it's recorded length of " << supposedSize << " does not match is actual length of " << (end - start) << '.' << endl;
                    }
                } else {
                    cerr << "Warning: Object maker returned NULL trying to decode a class; returning NULL back from the decoding";
                }
            }
    }
    
    return result;
}



Object::Object() : retainCount(1) {
    //cout << "Created " << (int)this << endl;
    
    typeCapacity = 4;
    objectTypes = (unsigned int*)malloc(sizeof(unsigned int) * typeCapacity);
    
    numberOfTypes = 1;
    objectTypes[0] = OBJECT;
}

void Object::getDescription(ostream &out) const {
    out << "Retain count: " << retainCount << '\n';
}

void Object::encode(Coder *coder) const {
    // Do nothing
}

bool Object::decode(Decoder *decoder) {
    return true;
}
