//  VectorAdditions.h
//
//  IMPORTANT: This code was developed by Wade Tregaskis under the employment of the Department of Primary Industries, Victoria, whom reserve all rights to it.  It has been used in good faith by Wade Tregaskis without explicit permission, but with the understanding from the employer that it's private use is acceptable in good faith.
//
//  The underlying message being, don't give this to anyone please as I don't have the rights for it's distribution.  Also, although I did write every character of it, I did so outside of university and as such I understand it may not be eligible for inclusion in university marking and other evaluation.

#ifndef VECTOR_ADDITIONS_H
#define VECTOR_ADDITIONS_H

#include <vector>
using namespace std;


/*! @header Vector Additions
    @abstract Provides a few additional functions for working with STL vectors.
    @discussion Stupidly, the STL list doesn't include functions for checking if a vector contains an object by reference to the object, nor for removing from the vector by reference to the object.  Thus, equivalent methods are implemented here.
    */
    
    
/*! @function vectorContains
    @abstract Determines whether a given vector contains a given object.
    @param v The vector.
    @param object The object.
    @result Returns true if the given vector contains the given object, false otherwise. */

template<class T> bool vectorContains(const vector<T> *v, const T object) {
    typename vector<T>::const_iterator iter, stopper;
    
    for (iter = v->begin(), stopper = v->end(); iter != stopper; ++iter) {
        if (*iter == object) {
            return true;
        }
    }
    
    return false;
}

/*! @function remove
    @abstract Removes an object from a given vector.
    @discussion Searches through the vector 'v' for all occurances of 'object', removing them as it goes.
    @param v The vector search through.
    @param object The object to remove. */

template<class T> void remove(vector<T> *v, const T object) {
    typename vector<T>::iterator iter = v->begin();
    
    while (iter != v->end()) {
        if (*iter == object) {
            iter = v->erase(iter);
        } else {
            ++iter;
        }
    }
}

#endif
